@@ -19,11 +19,12 @@ from djaq import app_utils
 
 logger = logging.getLogger(__name__)
 
+#  import ipdb
 
 """
 settings:
 
-DJAQ_WHITELIST
+DJAQ_WHITELIST or DJAQ_ALLOW_MODELS
 DJAQ_UI_URL
 DJAQ_API_URL
 DJAQ_PERMS = {
@@ -48,7 +49,7 @@ def is_user_allowed(user):
     return True
 
 
-def is_allowed(op):
+def has_permission(op):
     """Return True if op is allowed else False."""
     if hasattr(settings, "DJAQ_PERMISSIONS"):
         return settings.DJAQ_PERMISSIONS.get(op)
@@ -65,18 +66,21 @@ def get_validator():
 def get_whitelist():
     if hasattr(settings, "DJAQ_WHITELIST"):
         return settings.DJAQ_WHITELIST
-    return []
+    elif hasattr(settings, "DJAQ_ALLOW_MODELS"):
+        return settings.DJAQ_ALLOW_MODELS
+    return list()
 
 
 def queries(query_list, whitelist=None, validator=None):
+
     if not query_list:
-        return []
-    responses = []
+        return list()
+    responses = list()
     for data in query_list:
         query_string = data.get("q")
         offset = int(data.get("offset", 0))
         limit = int(data.get("limit", 0))
-        context = data.get("context", {})
+        context = data.get("context", dict())
         responses.append(
             list(
                 DQ(query_string, whitelist=whitelist)
@@ -91,11 +95,11 @@ def queries(query_list, whitelist=None, validator=None):
 
 
 def creates(creates_list, whitelist=None):
-    if not is_allowed("creates"):
+    if not has_permission("creates"):
         return Exception("Creates not allowed")
     if not creates_list:
-        return []
-    responses = []
+        return list()
+    responses = list()
     for data in creates_list:
         model = app_utils.find_model_class(data.pop("_model"), whitelist=whitelist)
         instance = model.objects.create(**data)
@@ -104,7 +108,7 @@ def creates(creates_list, whitelist=None):
 
 
 def updates(updates_list, whitelist=None):
-    if not is_allowed("updates"):
+    if not has_permission("updates"):
         raise Exception("Updates not allowed")
     if not updates_list:
         return []
@@ -117,11 +121,11 @@ def updates(updates_list, whitelist=None):
 
 
 def deletes(deletes_list, whitelist=None):
-    if not is_allowed("deletes"):
+    if not has_permission("deletes"):
         raise Exception("Deletes not allowed")
     if not deletes_list:
-        return []
-    responses = []
+        return list()
+    responses = list()
     for data in deletes_list:
         model = app_utils.find_model_class(data.pop("_model"), whitelist=whitelist)
         cnt = model.objects.filter(pk=data.pop("_pk")).delete()
@@ -129,11 +133,22 @@ def deletes(deletes_list, whitelist=None):
     return responses
 
 
+def get_context_data(data) -> dict:
+    """Look for 'context' item in 'queries' item."""
+    if "queries" in data:
+        if "context" in data["queries"]:
+            if isinstance(data["queries"], list):
+                return data["queries"][0]["context"]
+            else:
+                return data["queries"]["context"]
+    return dict()
+
+
 @csrf_exempt
 @login_required
 def djaq_request_view(request):
     """Main view for query and update requests."""
-    print(request.body)
+
     data = json.loads(request.body.decode("utf-8"))
     logger.debug(data)
 
@@ -144,39 +159,29 @@ def djaq_request_view(request):
 
     validator = get_validator()
 
-    #  import pudb; pudb.set_trace()
-    #  import ipdb; ipdb.set_trace()
-
-    # add request to context
-    q = data.get("queries")
-    ctx = dict()
-    try:
-        ctx = q[0].get("context")
-        if not ctx:
-            ctx = dict()
-    except Exception as e:
-        print(e)
-        
-    #  ctx["request"] = request
-    #  q["context"] = ctx
-
+    q = data.get("queries", dict()) or dict()
+    ctx = get_context_data(data)
+    # ctx["request"] = request
 
     try:
         queries_result = queries(q, whitelist=whitelist, validator=validator)
-        creates_result = \
-            creates(data.get("creates"), whitelist=whitelist) \
-            if is_allowed("creates") \
-            else []
+        creates_result = (
+            creates(data.get("creates"), whitelist=whitelist)
+            if has_permission("creates")
+            else list()
+        )
 
-        updates_result = \
-            updates(data.get("updates"), whitelist=whitelist) \
-            if is_allowed("updates") \
-            else []
+        updates_result = (
+            updates(data.get("updates"), whitelist=whitelist)
+            if has_permission("updates")
+            else list()
+        )
 
-        deletes_result = \
-            deletes(data.get("deletes"), whitelist=whitelist) \
-            if is_allowed("deletes") \
-            else []
+        deletes_result = (
+            deletes(data.get("deletes"), whitelist=whitelist)
+            if has_permission("deletes")
+            else list()
+        )
 
         return JsonResponse(
             {
@@ -206,6 +211,8 @@ def djaq_schema_view(request):
         return HttpResponse("Djaq unauthorized", status=401)
     if hasattr(settings, "DJAQ_WHITELIST"):
         whitelist = settings.DJAQ_WHITELIST
+    elif hasattr(settings, "DJAQ_ALLOW_MODELS"):
+        whitelist = settings.DJAQ_ALLOW_MODELS
     try:
         return JsonResponse(app_utils.get_schema(whitelist=whitelist))
     except Exception as e:
